#region Copyright Syncfusion Inc. 2001 - 2005
		//
		//  Copyright Syncfusion Inc. 2001 - 2005. All rights reserved.
		//
		//  Use of this code is subject to the terms of our license.
		//  A copy of the current license can be obtained at any time by e-mailing
		//  licensing@syncfusion.com. Re-distribution in any form is strictly
		//  prohibited. Any infringement will be prosecuted under applicable laws. 
		//
#endregion

#region file using directives
		using System;
		using System.Collections;
		using System.ComponentModel;
		using System.Drawing;
		using System.Data;
		using System.Windows.Forms;
		using System.Diagnostics;
		using System.Text;
		using Syncfusion.XlsIO;
		using Syncfusion.XlsIO.Implementation;
		using Syncfusion.Windows.Forms.Grid;
		using Syncfusion.Drawing;
		using Syncfusion.Grouping;
		using Syncfusion.GridExcelConverter;

		using ISummary = Syncfusion.Collections.BinaryTree.ITreeTableSummary;
#if ASPNET
using System.Web;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using Syncfusion.Web.UI.WebControls.Grid.Grouping;
#else 
		using Syncfusion.Windows.Forms.Grid.Grouping;
#endif
		#endregion

		namespace GroupingGrid
		{
			/// <summary>
			/// This controls provides support for Exporting datas from a GroupingGridControl into an Excel spreadsheet
			/// for verification and/or computation. This Control automatically Copies the Grid's Styles, Formats 
			/// to Excel. 
			/// The GroupingGridExcelConverter Control  is derived from 
			///<see cref="GridExcelConverterBase"/>.
			/// </summary>
			[ToolboxItem(false)]
			public class GroupingGridExcelConverterControlExt : GridExcelConverterBase
			{
				#region Class constants
				/// <summary>
				/// 
				/// </summary>
				private static readonly DisplayElementKind[] DEF_ADJUSTGROUPLEVELIFEQUAL = new DisplayElementKind[]
	{
		DisplayElementKind.Caption,
		DisplayElementKind.Table,
		DisplayElementKind.NestedTable,
				};
				#endregion

				#region Export Grouping Grid to Excel methods
				/// <summary>
				/// Converts grouping grid into excel file.
				/// </summary>
				/// <param name="grouping">Source grouping grid.</param>
				/// <param name="strFileName">Destination worksheet.</param>
				/// <param name="options">Convert options.</param>
				
				GridGroupingControl gdBase;
				public void GroupingGridToExcel( GridGroupingControl grouping, string strFileName,
					ConverterOptions options )
				{
					if( grouping == null )
						throw new ArgumentNullException( "grouping" );

					if( strFileName == null )
						throw new ArgumentNullException( "strFileName" );

					if( strFileName.Length == 0 )
						throw new ArgumentException( "strFileName - string can not be empty" );

					gdBase=grouping;


					ExcelEngine engine = CreateEngine();
					IWorkbook book = engine.Excel.Workbooks.Create( 1 );
					IWorksheet sheet = book.Worksheets[ 0 ];

					GroupingGridToExcel( grouping, sheet, options );

					book.Close( true, strFileName );
				}
				/// <summary>
				/// Converts grouping grid into excel file.
				/// </summary>
				/// <param name="grouping">Source grouping grid.</param>
				/// <param name="sheet">Destination worksheet.</param>
				/// <param name="options">Convert options.</param>
				public void GroupingGridToExcel( GridGroupingControl grouping, IWorksheet sheet,
					ConverterOptions options )
				{
					if( grouping == null )
						throw new ArgumentNullException( "grouping" );

					if( sheet == null )
						throw new ArgumentNullException( "sheet" );

					sheet.PageSetup.IsSummaryRowBelow = false;
					GridTable table = grouping.Table;

					if( ( options & ConverterOptions.Visible ) != 0 )
					{
						ExportElements( table.DisplayElements, sheet, 0, options, 1 );
					}
					else
					{
						ElementsInTableCollection arrElements = table.Elements;
						ExportElements( arrElements, sheet, 0, options, 1 );
					}
					GridTableModel tableModel = null;
#if ASPNET
      tableModel = grouping.Table.TableModel;
#else
					tableModel = grouping.TableModel;
#endif

					CopyColumnWidthFromGrid( tableModel, sheet, tableModel.GetColumnIndentCount() );
				}
				/// <summary>
				/// Exports collection of elements into worksheet.
				/// </summary>
				/// <param name="arrElements">Collection of elements to export.</param>
				/// <param name="sheet">Destination worksheet.</param>
				/// <param name="index">Starting zero-based index to the first row.</param>
				/// <param name="options">Convert options.</param>
				/// <returns>Index to the row after last exported element.</returns>
				private int ExportElements( IList arrElements, IWorksheet sheet, int index,
					ConverterOptions options, int iGroupLevel )
				{
					if( arrElements == null )
						throw new ArgumentNullException( "arrElements" );

					if( sheet == null )
						throw new ArgumentNullException( "worksheet" );

					if( index < 0 )
						throw new ArgumentOutOfRangeException( "index", index, "Value can not be less 0" );

					int iCount = arrElements.Count;

					if( iCount == 0 ) return 0;

					Element elementFirst = ( Element )arrElements[ 0 ];
					GridTableDescriptor tableDesc = ( GridTableDescriptor )elementFirst.ParentTable.TableDescriptor;
					int iColumnCount = tableDesc.Columns.Count;

					//for( int i = 0, len = arrElements.Count; i < len; i++ )
					int iIndexInGroup = 0;
					//int i = 0;

					Stack stackStartIndexes = new Stack();
					int iCurLevel = -1;
					int iRow = 0;
					int iStartRow = iRow;
					int iSkipCount = 0;

					foreach( Element element in arrElements )
					{
						if( iSkipCount > 0 )
						{
							iSkipCount--;
							continue;
						}

						switch( element.Kind )
						{
							case DisplayElementKind.Table:
								GridTable table = ( GridTable )element;

								IList elements = ( ( options & ConverterOptions.Visible ) != 0 )
									? elements = table.DisplayElements
									: elements = table.Elements;

								iRow += ExportElements( elements, sheet, index + iRow, options, iGroupLevel );
								break;

							case DisplayElementKind.Empty:
								GridEmptySection empty = ( GridEmptySection )element;
								iRow += ExportEmptyCells( sheet, index + iRow, options,
									empty.Appearance.EmptyCell, iColumnCount, iGroupLevel );
								break;

							case DisplayElementKind.AddNewRecord:
								RecordRow record = ( RecordRow )element;
								iRow += ExportRecordRow( record, sheet, index + iRow, options,
									iIndexInGroup, iGroupLevel, out iSkipCount );
								break;

							case DisplayElementKind.Record:
								iRow += ExportRecordRow( ( RecordRow )element, sheet, index + iRow, options, 
									iIndexInGroup, iGroupLevel, out iSkipCount );
								iIndexInGroup++;
								break;

							case DisplayElementKind.Caption:
																
								iIndexInGroup = 0;
								GridCaptionRow captionRow = ( GridCaptionRow )element;
								   
								iRow += ExportCaption( captionRow, sheet, index + iRow, options, 
									iGroupLevel, stackStartIndexes, ref iCurLevel );
								bool bVisible = ( ( ConverterOptions.Visible & options ) != 0 );

								if( bVisible && !captionRow.ParentChildTable.IsExpanded )
									return iRow - iStartRow;

								break;

							case DisplayElementKind.Summary:
								GridSummaryRow summary = ( GridSummaryRow )element;
								GridGroupOptionsStyleInfo info = summary.ParentTableDescriptor.ChildGroupOptions;
								if(! ( info.ShowCaptionSummaryCells 
									&& info.CaptionSummaryRow == summary.SummaryRowDescriptor.Name ))
								{
									iRow += ExportSummaryRow( summary, sheet, index + iRow, options, iGroupLevel );
								}
								
								break;

							case DisplayElementKind.ColumnHeader:
								GridColumnHeaderRow headerRow = ( GridColumnHeaderRow )element;
								iRow += ExportColumnHeader( headerRow, sheet, element, index + iRow, options, iGroupLevel, out iSkipCount );
								break;

							case DisplayElementKind.None:
							case DisplayElementKind.FilterBar:
							case DisplayElementKind.GroupHeader:
							case DisplayElementKind.GroupFooter:
							case DisplayElementKind.GroupPreview:
								iRow++;
								sheet.Range[ index + iRow, 1 ].Text = element.Kind.ToString();
								sheet.Range[ index + iRow, 2 ].Text = element.Info;
								break;

							case DisplayElementKind.RecordPreview:
								GridRecordPreviewRow previewRow = ( GridRecordPreviewRow )element;
								iRow++;
								sheet.Range[ index + iRow, 1 ].Text = element.Kind.ToString();
								sheet.Range[ index + iRow, 2 ].Text = element.Info;
								break;
						}
					}
					//Needed if you wants to add the Groups in a ExcelSheet....
//					if (iCurLevel > 0)
//					{
//						AdjustGroupingLevel(stackStartIndexes, ref iCurLevel, 0, sheet, index + iRow, false,
//							!elementFirst.GetVisibleInHierarchy());
//					}
					return iRow - iStartRow;
				}
				/// <summary>
				/// Exports column header.
				/// </summary>
				/// <param name="headerRow">Header row to export.</param>
				/// <param name="sheet">Destination worksheet.</param>
				/// <param name="element">Element to export.</param>
				/// <param name="index">One-based row index.</param>
				/// <param name="options">Convert options.</param>
				/// <param name="iGroupLevel">Group level - one-based column index.</param>
				/// <param name="iSkipCount">Number of elements to skip.</param>
				/// <returns>Number of exported (used) rows.</returns>
				private int ExportColumnHeader( GridColumnHeaderRow headerRow, IWorksheet sheet,
					Element element, int index, ConverterOptions options, int iGroupLevel, out int iSkipCount )
				{
					if( headerRow == null )
						throw new ArgumentNullException( "headerRow" );

					if( sheet == null )
						throw new ArgumentNullException( "sheet" );

					if( index < 0 )
						throw new ArgumentOutOfRangeException( "index", index, "Value can not be less 0" );
					GridTableCellStyleInfo style = null;
					int iRowIndex = index + 1;

					GridTableDescriptor table = ( GridTableDescriptor )headerRow.ParentTableDescriptor;
					GridColumnDescriptorCollection arrColumns = table.Columns;
					GridVisibleColumnDescriptorCollection arrVisibleColumns = table.VisibleColumns;
					GridColumnSetDescriptorCollection arrColumnSets = table.ColumnSets;

					GridTableOptionsStyleInfo tableStyleInfo = table.TableOptions;
					sheet.SetRowHeightInPixels( iRowIndex, tableStyleInfo.ColumnHeaderRowHeight );
					style = table.Appearance.ColumnHeaderCell;

                    
                    
					int iRowCount = 1;

					for( int i = 0, iColumnIndex = iGroupLevel, len = arrVisibleColumns.Count; i < len; i++, iColumnIndex++ )
					{
						GridVisibleColumnDescriptor column = arrVisibleColumns[ i ];
						string strColumnName = column.Name;

						if( arrColumnSets.Contains( strColumnName ) )
						{
#if ASPNET
          GridColumnSetDescriptor columnSet = arrColumnSets.GetColumnSetDescriptor( strColumnName );
#else
							GridColumnSetDescriptor columnSet = arrColumnSets[ strColumnName ];
#endif
							int iRows = ExportColumnSet( arrColumns, columnSet, sheet, iRowIndex, ref iColumnIndex, style );

							if( iRows > iRowCount ) iRowCount = iRows;
						}
						else
						{
							IRange range = sheet.Range[ iRowIndex, iColumnIndex ];
							//Lan: Should use the visiblecolumns
							//range.Text = arrColumns[column.Name].HeaderText;
							if( arrColumns.Contains(column.Name)) 
								range.Text = arrColumns[column.Name].HeaderText;
							else
								range.Text = column.Name;

                            GridTable gtable = element.ParentTable as GridTable;
                            style = gtable.GetTableCellStyle(element, column.Name);
							CopyStyle( style, range );
						}
					}

					iSkipCount = iRowCount - 1;
					return iRowCount;
				}

				/// <summary>
				/// Exports column set into excel.
				/// </summary>
				/// <param name="arrColumns">Columns collection.</param>
				/// <param name="columnSet">Column set to export.</param>
				/// <param name="sheet">Worksheet to export data into.</param>
				/// <param name="iRowIndex">Row index to export to.</param>
				/// <param name="iColumnIndex">Column index to export to.</param>
				/// <param name="style">Style of the exported cells.</param>
				/// <returns>Number of rows used by column set.</returns>
				private int ExportColumnSet( GridColumnDescriptorCollection arrColumns, 
					GridColumnSetDescriptor columnSet, IWorksheet sheet,
					int iRowIndex, ref int iColumnIndex, GridStyleInfo style )
				{
					GridColumnSpanDescriptorCollection arrColumnSpans = columnSet.ColumnSpans;
					int iRowCount = 1;
					int iMaxColumn = 0;

					for( int i = 0, len = arrColumnSpans.Count; i < len; i++ )
					{
						GridColumnSpanDescriptor columnSpan = arrColumnSpans[ i ];
						GridRangeInfo gridRange = columnSpan.Range;

						int iSpanRowIndex = iRowIndex + gridRange.Top;
						int iSpanColumnIndex = iColumnIndex + gridRange.Left;

						IRange range = sheet.Range[ iSpanRowIndex, iSpanColumnIndex ];
						string strColumnName = columnSpan.Name;

#if ASPNET
        range.Text = arrColumns.GetColumnDescriptor( strColumnName ).HeaderText;
#else
						range.Text = arrColumns[ strColumnName ].HeaderText;
#endif

						sheet.Range[ iSpanRowIndex, iSpanColumnIndex,
							iRowIndex + gridRange.Bottom, iColumnIndex + gridRange.Right ].Merge();

						CopyStyle( style, range );

						iRowCount = Math.Max( gridRange.Bottom + 1, iRowCount );
						iMaxColumn = Math.Max( gridRange.Right, iMaxColumn );
					}

					iColumnIndex += iMaxColumn;
					return iRowCount;
				}
				/// <summary>
				/// Exports empty cells.
				/// </summary>
				/// <param name="sheet">Sheet to export cells into.</param>
				/// <param name="index">Zero-based row index.</param>
				/// <param name="options">Converter options.</param>
				/// <param name="style">Style of the cells to export.</param>
				/// <param name="iColumnCount">Number of columns to export.</param>
				/// <param name="iGroupLevel">Group level.</param>
				/// <returns>Number of used rows.</returns>
				private int ExportEmptyCells( IWorksheet sheet, int index,
					ConverterOptions options, GridTableCellStyleInfo style, int iColumnCount,
					int iGroupLevel )
				{
					if( sheet == null )
						throw new ArgumentNullException( "sheet" );

					if( index < 0 )
						throw new ArgumentOutOfRangeException( "index", index, "Value can not be less 0" );

					IRange usedRange = sheet.Range;
					int iRowIndex = index + 1;

					if( iColumnCount > 0 )
					{
						IRange range = sheet.Range[ iRowIndex, iGroupLevel, iRowIndex, iColumnCount + iGroupLevel - 1 ];
						CopyStyle( style, range );
					}

					sheet.SetRowHeightInPixels( iRowIndex, 0 );
					return 1;
				}
				/// <summary>
				/// Exports summary row into excel worksheet.
				/// </summary>
				/// <param name="summary">Summary to export.</param>
				/// <param name="sheet">Destination worksheet.</param>
				/// <param name="index">Zero-based index to first row of the summary.</param>
				/// <param name="options">Converter options.</param>
				private int ExportSummaryRow( GridSummaryRow summary, IWorksheet sheet, int index,
					ConverterOptions options , int iGroupLevel )
				{
					if( summary == null )
						throw new ArgumentNullException( "summary" );

					if( sheet == null )
						throw new ArgumentNullException( "sheet" );

					if( index < 0 )
						throw new ArgumentOutOfRangeException( "index", index, "Value can not be less 0" );

					int iRow = index + 1;
					GridSummaryRowDescriptor descriptor = summary.SummaryRowDescriptor;
//					GridTable table = summary.ParentTable as GridTable; 
//					int RowIndex = table.DisplayElements.IndexOf(summary);  
//					table.GetTableCellStyle(RowIndex,1);  

					IRange range = sheet.Range[ iRow, iGroupLevel ];
					
					//Lan: Indent group captions
					//range.Text = captionText ;//+ GetCaptionText( captionRow );
					//range.Text = descriptor.Title;
					if (summary.GroupLevel==0) //no group
					{
						range.Text ="Grand "+descriptor.Title;
					}
					else
					{
						int iTotal=descriptor.Title.Length+(summary.GroupLevel-1)*INDENT;
						range.Text =descriptor.Title.PadLeft(iTotal);
					}

					CopyStyle( descriptor.Appearance.SummaryTitleCell, range );

					GridTableDescriptor table = summary.ParentTableDescriptor;
					GridSummaryColumnDescriptorCollection arrColumns = descriptor.SummaryColumns;

					int iColumnsCount = table.Columns.Count;
					range = sheet.Range[ iRow, iGroupLevel + 1, iRow, iGroupLevel + iColumnsCount - 1 ];
					CopyStyle( descriptor.Appearance.SummaryEmptyCell, range );
					  

					GridTableOptionsStyleInfo tableStyleInfo = table.TableOptions;
					sheet.SetRowHeightInPixels( iRow, tableStyleInfo.RecordRowHeight );
					

					for( int i = 0, len = arrColumns.Count; i < len; i++ )
					{
						GridSummaryColumnDescriptor column = arrColumns[ i ];


						int iColumnIndex = column.ColInRecord;
						GridTableCellStyleInfo style = null;

						// lan: Skip for the Title cell:range = sheet.Range[ iRow, iGroupLevel ]
						// because it has been setup in code before the loop
//						if( iColumnIndex == -1 )
//						{
//							range = sheet.Range[ iRow, iGroupLevel ];
//						}
//						else
//						{
//								range = sheet.Range[ iRow, iGroupLevel + iColumnIndex ];
//								style = table.Appearance.SummaryFieldCell;
//								CopyStyle( style, range );
//						}
//						range.Text = column.GetDisplayText( summary.ParentGroup );
						if( iColumnIndex >0 )
						{
							
							range = sheet.Range[ iRow, iGroupLevel + iColumnIndex ];
							style = table.Appearance.SummaryFieldCell;
							CopyStyle( style, range );
							//range.Text = column.GetDisplayText( summary.ParentGroup );
							range.Value=column.GetDisplayText( summary.ParentGroup );

						}
						

					}


					return 1;
				}
				/// <summary>
				/// Performs all necessary operations (grouping necessary rows, etc.)
				/// to set current group level to the desired level.
				/// </summary>
				/// <param name="stackStartIndexes">Stack with starting index of previously found groups.</param>
				/// <param name="iCurLevel">Current group level.</param>
				/// <param name="iGroupLevel">Group level to set.</param>
				/// <param name="sheet">Destination worksheet.</param>
				/// <param name="index">Current row zero-based index.</param>
				/// <param name="bStartNewGroup">Indicates whether new group should be started at index.</param>
				private void AdjustGroupingLevel(Stack stackStartIndexes, ref int iCurLevel, int iGroupLevel,
					IWorksheet sheet, int index,  bool bStartNewGroup,
					bool expanded) 
				{
					if (stackStartIndexes == null) 
					{
						throw new ArgumentNullException("stackStartIndexes");
					}

					if (sheet == null) 
					{
						throw new ArgumentNullException("sheet");
					}

					if (iCurLevel < 0 || iCurLevel < iGroupLevel) 
					{
						iCurLevel = iGroupLevel;
						stackStartIndexes.Push(new GroupInfo(index, expanded));
					} 
					else if (iCurLevel >= iGroupLevel) 
					{
						int iLastLevel = bStartNewGroup ? iGroupLevel : iGroupLevel + 1;

						for (int lC = iCurLevel - iGroupLevel; iCurLevel >= iLastLevel; iCurLevel--) 
						{
							GroupInfo gi = (GroupInfo) stackStartIndexes.Pop();
							int iStartIndex = gi.StartIndex ;

							int iEndIndex = index;
							sheet.Range[iStartIndex + 1, 1, index, 1].Group(ExcelGroupBy.ByRows, !gi.Expanded);

						}

						if (bStartNewGroup) 
						{
							iCurLevel++;
							stackStartIndexes.Push(new GroupInfo(index,  expanded));
						}
					}
				}

				public class GroupInfo 
				{
					private int _indx;
					private bool _expanded;

					public GroupInfo(int startIndex, bool expanded) 
					{
						_indx = startIndex + 1;
						_expanded = expanded;
					}

					public int StartIndex 
					{
						get { return _indx; }
					}

					public bool Expanded 
					{
						get { return _expanded; }
					}

				}
				/// <summary>
				/// 
				/// </summary>
				/// <param name="row"></param>
				/// <param name="sheet"></param>
				/// <param name="index"></param>
				/// <param name="options"></param>
				/// <param name="iIndexInGroup"></param>
				/// <returns></returns>
				private int ExportRecordRow( RecordRow row, IWorksheet sheet, int index, ConverterOptions options
					, int iIndexInGroup, int iGroupLevel, out int iSkipRecordRows )
				{
					if( row == null )
						throw new ArgumentNullException( "row" );

					if( sheet == null )
						throw new ArgumentNullException( "sheet" );

					if( index < 0 )
						throw new ArgumentOutOfRangeException( "index", index, "Value can not be less 0" );

					if( iIndexInGroup < 0 )
						throw new ArgumentOutOfRangeException( "iIndexInGroup", iIndexInGroup, "Value can not be less than 0" );

					FieldDescriptorCollection arrFields = row.ParentTable.TableDescriptor.Fields;
					int iRowIndex = index + 1;

					GridTableDescriptor table = ( GridTableDescriptor )row.ParentTableDescriptor;
					//GridColumnDescriptorCollection arrColumns = table.Columns;
					GridVisibleColumnDescriptorCollection arrVisibleColumns = table.VisibleColumns;
					GridColumnSetDescriptorCollection arrColumnSets = table.ColumnSets;

					GridTableOptionsStyleInfo tableStyleInfo = table.TableOptions;
					sheet.SetRowHeightInPixels( iRowIndex, tableStyleInfo.RecordRowHeight );

					GridTable t = row.ParentTable as GridTable;

					GridTableCellStyleInfo style = ( iIndexInGroup % 2 == 0 )
						? table.Appearance.RecordFieldCell
						: table.Appearance.AlternateRecordFieldCell;

					int iRowCount = 1;

					for( int i = 0, len = arrVisibleColumns.Count; i < len; i++ )
					{
						int iColumnIndex = iGroupLevel + i;
						GridVisibleColumnDescriptor column = arrVisibleColumns[ i ];
						string strColumnName = column.Name;
						
						//lan: is combobox
						bool bIsComboBox=(table.Columns[strColumnName].Appearance.AnyRecordFieldCell.CellType =="ComboBox");

						if( arrColumnSets.Contains( strColumnName ) )
						{
#if ASPNET
          GridColumnSetDescriptor columnSet = arrColumnSets.GetColumnSetDescriptor( strColumnName );
#else
							GridColumnSetDescriptor columnSet = arrColumnSets[ strColumnName ];
#endif
							int iUsedRows = ExportColumnSet( columnSet, sheet, iRowIndex,
								ref iColumnIndex, style, arrFields, row );

							if( iUsedRows > iRowCount ) iRowCount = iUsedRows;
						}
						else
						{
							FieldDescriptor field = arrFields[ strColumnName ];
          
							IRange range = sheet.Range[ iRowIndex, iColumnIndex ];
							
							if( row.ParentTable != null)
							{
								int RowIndex = row.ParentTable.DisplayElements.IndexOf(row);
								int ColIndex = table.NameToField( strColumnName );  
								ColIndex = table.FieldToColIndex( ColIndex );  
								GridEngine engine = row.ParentTable.Engine as GridEngine;      
								GridTableControl tc = engine.GroupingControl.GetTableControl(table.Name);  


								//Lan: Instead of using the GetTableCellStyleInfo method, try using the TableModel indexer 
								//to get the cell style in a grid. 
								//GridTableCellStyleInfo cellStyle = tc.GetTableViewStyleInfo(RowIndex,ColIndex); 
								GridTableCellStyleInfo cellStyle = t.TableModel[RowIndex,ColIndex] ;
									
								//if(cellStyle.CellType == "ComboBox") 
								if (bIsComboBox)
								{
									//range.Value2 = cellStyle.FormattedText;
									//Lan: To display combobox lookup desc,
									//must use the visiblecolumn index because when it is grouped, col index changed
									ColIndex = table.VisibleColumns.IndexOf( strColumnName );  
									ColIndex = table.FieldToColIndex( ColIndex );  


									range.Value2 =  t.TableModel[RowIndex,ColIndex].FormattedText;


								}
								else
								{
									object value = row.ParentRecord.GetValue( field );
									range.Value2 = ( value == null ) ? string.Empty : value;
								}
								CopyStyle( cellStyle, range );
							}
						}
					}

					iSkipRecordRows = iRowCount - 1;

					Record rec = Element.GetRecord( row );//Record rec = row.ParentRecord;
					bool bVisible = ( ( ConverterOptions.Visible & options ) != 0 );

					if( bVisible & !rec.IsExpanded ) return iRowCount;

					NestedTablesCollection nestedTables = rec.NestedTables;
					int iCount = nestedTables.Count;

					if( iCount > 0 ) iGroupLevel++;

					for( int i = 0; i < iCount; i++ )
					{
						//        NestedTable nestedTable = nestedTables[ i ];
						//        IList elements = nestedTable.ChildTable.NestedDisplayElements;
						//        iRowCount += ExportElements( elements, sheet, index + iRowCount, options );
						GridNestedTable nestedTable = ( GridNestedTable )nestedTables[ i ];

						IList elements = nestedTable.ChildTable.Elements;
						int ix = index+iRowCount;
						iRowCount += ExportElements( elements, sheet, index + iRowCount, options, iGroupLevel );
						sheet.Range[ix+1, 1, index + iRowCount, 1].Group(ExcelGroupBy.ByRows, !rec.IsExpanded);
					}

					return iRowCount;
				}
				/// <summary>
				/// Exports column set into excel.
				/// </summary>
				/// <param name="arrColumns">Columns collection.</param>
				/// <param name="columnSet">Column set to export.</param>
				/// <param name="sheet">Worksheet to export data into.</param>
				/// <param name="iRowIndex">Row index to export to.</param>
				/// <param name="iColumnIndex">Column index to export to.</param>
				/// <param name="style">Style of the exported cells.</param>
				/// <returns>Number of rows used by column set.</returns>
				private int ExportColumnSet( GridColumnSetDescriptor columnSet, IWorksheet sheet,
					int iRowIndex, ref int iColumnIndex, GridStyleInfo style,
					FieldDescriptorCollection arrFields, RecordRow row )
				{
					GridColumnSpanDescriptorCollection arrColumnSpans = columnSet.ColumnSpans;
					Record parentRecord = row.ParentRecord;
					int iRowCount = 1;
					int iMaxColumn = 0;

					for( int i = 0, len = arrColumnSpans.Count; i < len; i++ )
					{
						GridColumnSpanDescriptor columnSpan = arrColumnSpans[ i ];
						GridRangeInfo gridRange = columnSpan.Range;

						int iSpanRowIndex = iRowIndex + gridRange.Top;
						int iSpanColumnIndex = iColumnIndex + gridRange.Left;

						IRange range = sheet.Range[ iSpanRowIndex, iSpanColumnIndex ];
						string strColumnName = columnSpan.Name;
						FieldDescriptor field = arrFields[ strColumnName ];
						object value = parentRecord.GetValue( field );

						range.Value2 = ( value != null ) ? value : string.Empty;

						sheet.Range[ iSpanRowIndex, iSpanColumnIndex,
							iRowIndex + gridRange.Bottom, iColumnIndex + gridRange.Right ].Merge();

						CopyStyle( style, range );

						iRowCount = Math.Max( gridRange.Bottom + 1, iRowCount );
						iMaxColumn = Math.Max( gridRange.Right, iMaxColumn );
					}

					iColumnIndex += iMaxColumn;
					return iRowCount;
				}
				/// <summary>
				/// Export element style.
				/// </summary>
				/// <param name="sourceElement">Source element to export.</param>
				/// <param name="sourceField">Source field descriptor.</param>
				/// <param name="destRange">Destination range.</param>
				/// <param name="options">Convert options.</param>
				private void ExportStyle( Element sourceElement, FieldDescriptor sourceField, IRange destRange,
					ConverterOptions options )
				{
					if( destRange == null )
						throw new ArgumentNullException( "destRange" );

					if( sourceElement == null )
						throw new ArgumentNullException( "sourceElement" );

					GridTable table = sourceElement.ParentTable as GridTable;

					if( table == null )
					{
						Debug.WriteLine( "Bad table type" );
						return;
					}

					GridTableCellStyleInfo style;

				{
					string strFieldName = sourceField != null ? sourceField.Name : string.Empty;
					style = table.GetTableCellStyle( sourceElement, strFieldName );
				}


					CopyStyle( style, destRange );
				}

//				/// <summary>
//				/// Exports caption row.
//				/// </summary>
//				/// <param name="captionRow">Source caption row.</param>
//				/// <param name="sheet">Destionation worksheet.</param>
//				/// <param name="index">Capiton's zero-based row index in the worksheet.</param>
//				/// <param name="options">Convert options.</param>
//				/// <param name="iCurrentGroupLevel">Current group level.</param>
//				/// <returns>Zero-based row index that point to the row after caption.</returns>
//				private int ExportCaption( GridCaptionRow captionRow, IWorksheet sheet, int index,
//					ConverterOptions options, int iCurrentGroupLevel, Stack stackStartIndexes, ref int iCurrentLevel)
//				{
//					if( captionRow == null )
//						throw new ArgumentNullException( "captionRow" );
//
//					if( sheet == null )
//						throw new ArgumentNullException( "sheet" );
//
//					if( index < 0 )
//						throw new ArgumentOutOfRangeException( "index", index, "Value can not be less 0" );
//
//					IRange range = sheet.Range[ index + 1, iCurrentGroupLevel ];
//					range.Text = GetCaptionText( captionRow );
//					 
//					CopyStyle( captionRow.Appearance.GroupCaptionCell, range );
//					GridTableModel tableModel = captionRow.ParentTable.TableModel;
//					int iRealColCount = tableModel.ColCount - tableModel.GetColumnIndentCount();
//					int iRowIndex = index + 1;
//
//					GridTableDescriptor table = ( GridTableDescriptor )captionRow.ParentTableDescriptor;
//					GridTableOptionsStyleInfo tableStyleInfo = table.TableOptions;
//					sheet.SetRowHeightInPixels( iRowIndex, tableStyleInfo.CaptionRowHeight );
//
//					if( iRealColCount > 0 )
//					{
//						GridGroupOptionsStyleInfo info = table.ChildGroupOptions;
//						if( !captionRow.ParentGroup.IsMainGroup
//							&& table != null && info.ShowCaptionSummaryCells )  
//						{
//							GridSummaryRowDescriptor summary = table.SummaryRows[info.CaptionSummaryRow];
//							
//							int iFirstSummaryColIndex = iRealColCount;
//
//							for(int i = 0 ;i < iRealColCount; i++)
//							{	
//								GridSummaryColumnDescriptor col = summary.GetSummaryColumnAtCol(i); 
//								if( col != null )
//								{
//									if( iFirstSummaryColIndex == iRealColCount)
//										iFirstSummaryColIndex = i;
//									GridTableCellStyleInfo cellstyle = col.Appearance.GroupCaptionSummaryCell;
//									CopyStyle(cellstyle ,sheet.Range[iRowIndex,i+ 1]);  
//									sheet.Range[iRowIndex,i + 1 ].Text = col.GetDisplayText(captionRow.ParentGroup) ;
//								}
//								else
//								{
//									GridTableCellStyleInfo scellstyle = captionRow.Appearance.GroupCaptionSummaryCell;
//									CopyStyle(scellstyle ,sheet.Range[ iRowIndex, i + 1 ]);  
//								}
//							}
//
//							sheet.Range[ iRowIndex, iCurrentGroupLevel, iRowIndex, iCurrentGroupLevel + iFirstSummaryColIndex - 1 ].Merge();
//						}
//						else 
//						sheet.Range[ iRowIndex, iCurrentGroupLevel, iRowIndex, iCurrentGroupLevel + iRealColCount - 1 ].Merge();
//					}
//
//					int iGroupLevel = captionRow.GroupLevel;
//
////					AdjustGroupingLevel(stackStartIndexes, ref iCurrentLevel, iGroupLevel, sheet, index, true,
////						captionRow.ParentGroup.IsExpanded);
//
//					return 1;
//				}
				
				/// <summary>
				/// Exports caption row.
				/// </summary>
				/// <param name="captionRow">Source caption row.</param>
				/// <param name="sheet">Destionation worksheet.</param>
				/// <param name="index">Capiton's zero-based row index in the worksheet.</param>
				/// <param name="options">Convert options.</param>
				/// <param name="iCurrentGroupLevel">Current group level.</param>
				/// <returns>Zero-based row index that point to the row after caption.</returns>
				private const int INDENT=3;
				private int ExportCaption( GridCaptionRow captionRow, IWorksheet sheet, int index,
					ConverterOptions options, int iCurrentGroupLevel, Stack stackStartIndexes, ref int iCurrentLevel)
				{
					if( captionRow == null )
						throw new ArgumentNullException( "captionRow" );

					if( sheet == null )
						throw new ArgumentNullException( "sheet" );

					if( index < 0 )
						throw new ArgumentOutOfRangeException( "index", index, "Value can not be less 0" );

					//lan: 
					if(captionRow.ParentGroup.IsTopLevelGroup )
					{
						// never show the first row
						//if you set ShowCaption=false in TopLevelGroupOptions, 
						//then export the grid to excel. It will export the first column Headertext 'Identity'
						//in the first row of excel sheet, instead of the 'My Table 17 Items'.
						return 1;
					}

					IRange range = sheet.Range[ index + 1, iCurrentGroupLevel ];

					string captionText  =  string.Empty; 
					if( captionRow.ParentTable != null )
					{
						GridTable t = captionRow.ParentTable as GridTable; 
						for(int icol = 0 ;icol< 5;icol++)
						{
							GridTableCellStyleInfo style = null;//t.GetTableCellStyle(captionRow,icol );
							int iindex = t.DisplayElements.IndexOf(captionRow);   
							if( iindex != -1)
								style = t.TableModel[iindex,icol]; 
							if( style != null)
							{
								if( style.Text != string.Empty)
								{
									captionText = style.Text; 
									break;
								}
							}
						}
					}

					//Lan: Indent group captions
					//range.Text = captionText ;//+ GetCaptionText( captionRow );
					int iTotal=captionText.Length+(captionRow.GroupLevel-1)*INDENT;
					range.Text =captionText.PadLeft(iTotal);
					 
					CopyStyle( captionRow.Appearance.GroupCaptionCell, range );
					GridTableModel tableModel = captionRow.ParentTable.TableModel;
					int iRealColCount = tableModel.ColCount - tableModel.GetColumnIndentCount();
					int iRowIndex = index + 1;

					GridTableDescriptor table = ( GridTableDescriptor )captionRow.ParentTableDescriptor;
					GridTableOptionsStyleInfo tableStyleInfo = table.TableOptions;
					sheet.SetRowHeightInPixels( iRowIndex, tableStyleInfo.CaptionRowHeight );

					if( iRealColCount > 0 )
					{
						GridGroupOptionsStyleInfo info = table.ChildGroupOptions;
						if( !captionRow.ParentGroup.IsMainGroup
							&& table != null && info.ShowCaptionSummaryCells )  
						{
							GridSummaryRowDescriptor summary = table.SummaryRows[info.CaptionSummaryRow];
							
							int iFirstSummaryColIndex = iRealColCount;

							for(int i = 0 ;i < iRealColCount; i++)
							{	
								GridSummaryColumnDescriptor col = summary.GetSummaryColumnAtCol(i); 
								if( col != null )
								{
									if( iFirstSummaryColIndex == iRealColCount)
										iFirstSummaryColIndex = i;
									GridTableCellStyleInfo cellstyle = col.Appearance.GroupCaptionSummaryCell;
									
									CopyStyle(cellstyle ,sheet.Range[iRowIndex,i+ 1]);  
									sheet.Range[iRowIndex,i + 1 ].Text =  col.GetDisplayText(captionRow.ParentGroup) ;
								}
								else
								{
									GridTableCellStyleInfo scellstyle = captionRow.Appearance.GroupCaptionSummaryCell;
									CopyStyle(scellstyle ,sheet.Range[ iRowIndex, i + 1 ]);  
								}
							}

							sheet.Range[ iRowIndex, iCurrentGroupLevel, iRowIndex, iCurrentGroupLevel + iFirstSummaryColIndex - 1 ].Merge();
						}
						else 
							sheet.Range[ iRowIndex, iCurrentGroupLevel, iRowIndex, iCurrentGroupLevel + iRealColCount - 1 ].Merge();
					}

					int iGroupLevel = captionRow.GroupLevel;

					//					AdjustGroupingLevel(stackStartIndexes, ref iCurrentLevel, iGroupLevel, sheet, index, true,
					//						captionRow.ParentGroup.IsExpanded);

					return 1;
				}

				/// <summary>
				/// Exports group to the excel worksheet.
				/// </summary>
				/// <param name="group">Group to export.</param>
				/// <param name="sheet">Destination worksheet.</param>
				/// <param name="index">Zero-based row index.</Bparam>
				/// <returns>Zero-based row index that point to the row after group rows.</returns>
				private int ExportGroup( GridGroup group, IWorksheet sheet, int index, int iGroupLevel )
				{
					if( group == null )
						throw new ArgumentNullException( "group" );

					if( sheet == null )
						throw new ArgumentNullException( "sheet" );

					if( index < 0 )
						throw new ArgumentOutOfRangeException( "index", index, "Value can not be less 0" );

					RecordsInDetailsCollection arrRecords = group.Records;
					return ExportElements( arrRecords, sheet, index, ConverterOptions.Default, iGroupLevel );
				}
				/// <summary>
				/// Exports nested table.
				/// </summary>
				/// <param name="table">Source table to export.</param>
				/// <param name="sheet">Destination worksheet.</param>
				/// <param name="index">Capiton's zero-based row index in the worksheet.</param>
				/// <param name="options">Convert options.</param>
				/// <returns>Zero-based row index that point to the row after caption.</returns>
				private int ExportNestedTable( NestedTable table, IWorksheet sheet, int index,
					ConverterOptions options, int iGroupLevel )
				{
					IList elements = null;
					ChildTable childTable = table.ChildTable;

					elements = ( ( options & ConverterOptions.Visible ) != 0 )
						? ( IList )childTable.DisplayElements
						: ( IList )childTable.Elements;

					return ExportElements( elements, sheet, index, options, iGroupLevel );
				}
				#endregion

				#region Class static methods
				/// <summary>
				/// Retrieves caption text from caption row.
				/// </summary>
				/// <param name="caption">Caption row to get text from.</param>
				/// <returns>Caption text.</returns>
				public static string GetCaptionText( GridCaptionRow caption )
				{
					return GetGroupCaptionText( caption.ParentGroup );
				}
				/// <summary>
				/// Retrieves caption text for group.
				/// </summary>
				/// <param name="group">Group to get caption text from.</param>
				/// <returns>Caption text.</returns>
				public static string GetGroupCaptionText( Group group )
				{
					IGridGroupOptionsSource g = group as IGridGroupOptionsSource;
					string captionText;

					captionText = ( g != null )
						? g.GroupOptions.CaptionText
						: captionText = "{CategoryCaption}: {Category} - {RecordCount} Items";
 
					return GetGroupCaptionDisplayText( group, captionText );
				}

				/// <summary>
				/// Gets caption text for a group.
				/// </summary>
				/// <param name="group">The group to get caption text for.</param>
				/// <param name="format">Caption format.</param>
				/// <returns>Caption text for the group.</returns>
				public static string GetGroupCaptionDisplayText( Group group, string format )
				{
					GridTableDescriptor tableDescriptor = ( GridTableDescriptor )group.ParentTableDescriptor;

					bool raiseException = false;
					ArrayList al = new ArrayList();
					int iOpenBracketPos = format.IndexOf( "{" );
					StringBuilder sb = new StringBuilder();
					int iCloseBracketPos = 0;

					if( iOpenBracketPos == -1 )
					{
						sb.Append( format );
					}
					else
					{
						sb.Append( format.Substring( 0, iOpenBracketPos + 1 ) );
					}
 
					while( iOpenBracketPos != -1 )
					{
						iCloseBracketPos = format.IndexOf( "}", iOpenBracketPos );

						if( iOpenBracketPos != -1 && iCloseBracketPos != -1 && iCloseBracketPos > iOpenBracketPos )
						{
							int n3 = format.IndexOfAny( new char[] { '}', ':' }, iOpenBracketPos );
							string name = format.Substring( iOpenBracketPos + 1, n3 - iOpenBracketPos - 1 );
  
							sb.Append( al.Count.ToString() );
							object obj = "";

							if( name == "TableName" )
							{
								obj = group.ParentTableDescriptor.Name;
							}
							else if( name == "CategoryName" )
							{
								obj = group.Name;
							}
							else if( name == "CategoryCaption" )
							{
								string fieldName = group.Name;

								GridColumnDescriptor cd = group.GroupLevel >= 0
									? tableDescriptor.Columns.FindByMappingName( fieldName )
									: null;

								obj = ( cd != null ) ? cd.HeaderText : fieldName;
							}
							else if( name == "Category" )
							{
								obj = group.Category;
							}
							else if( name == "RecordCount" )
							{
								obj = group.GetFilteredRecordCount();
							}
							else
							{
								GridTableDescriptor td = ( ( GridTableDescriptor )group.ParentTableDescriptor );
								int summaryRowNum;
								int dot = name.IndexOf( '.' );

								if( dot != -1 )
								{
									string rowName = name.Substring( 0, dot );
									name = name.Substring( dot + 1 );
									summaryRowNum = td.SummaryRows.IndexOf(rowName);
								}
								else
								{
									summaryRowNum = td.SummaryRows.IndexOf( "GroupCaption" );
								}

								if( summaryRowNum != -1 )
								{
									GridSummaryColumnDescriptor scd=null;
#if ASPNET
       scd= td.SummaryRows[ summaryRowNum ].SummaryColumns.GetSummaryColDescriptor(name);
#else
									scd = td.SummaryRows[ summaryRowNum ].SummaryColumns[name];
#endif
									if( scd != null )
									{
										Group g = group;
										Syncfusion.Collections.BinaryTree.ITreeTableSummary[] sums =
											g.GetSummaries( group.ParentTable );
										int ndx = scd.GetSummaryIndex();
										obj = ( scd.GetDisplayText( sums[ ndx ] ) );
									}
								}
							}
							al.Add( obj );
							iOpenBracketPos = format.IndexOf( "{", iCloseBracketPos );

							if( iOpenBracketPos == -1 )
							{
								sb.Append( format.Substring( n3 ) );
							}
							else
							{
								sb.Append( format.Substring( n3, iOpenBracketPos - n3 + 1 ) );
							}
						}
						else
						{
							if( raiseException )
							{
								throw new FormatException( "No closing char found: " + sb.ToString() );
							}
							else
							{
								break;
							}
						}
					}
					string formatString = sb.ToString();

					try
					{
						return String.Format(formatString, al.ToArray());
					}
					catch (Exception ex)
					{
						return formatString + ": " + ex.Message;
					}
				}



				#endregion

				#region Class properties
				/// <summary>
				/// Indicates whether default alignment setting should be used when alignment
				/// wasn't set. Read-only.
				/// </summary>
				public override bool UseDefaultAlignment
				{
					get
					{
						return false;
					}
				}
				#endregion
			}
		}
